import json, subprocess

from app.config import ALERTS_DIR, MAIN_SCRIPT
from managed_servers import Service
from app.utils import Result
from app.utils.check_processing import ScriptStatus


def script_not_started(service: Service) -> bool:
    no_file = ": cannot open"

    cmd = f"file {service.background_data}"
    cmd_result = subprocess.Popen(
        [cmd], shell=True, stdout=subprocess.PIPE, universal_newlines=True
    ).communicate()[0]
    # If the background data file is missing, then the bash script was not started to run in the background earlier and
    # can be started now.
    if no_file in cmd_result:
        return True

    return False


def sort_data(cmd_result: str) -> dict:
    top_lvl_vars = ["auto_fix_status", "script_status", "force_report"]
    raw_data = json.loads(cmd_result)
    sorted_data = {"payload": {}}
    for k in raw_data.keys():
        if k in top_lvl_vars:
            sorted_data[k] = raw_data[k]
        else:
            sorted_data["payload"][k] = raw_data[k]

    return sorted_data


def run_service_check(service: Service) -> Result:
    service_check_result = Result()

    try:
        cmd = f"source {ALERTS_DIR}/{MAIN_SCRIPT}; {service.auto_fix_script}"
        if script_not_started(service):
            # start_script
            subprocess.Popen(
                [cmd], shell=True, stdout=subprocess.DEVNULL, close_fds=True
            )
            service_check_result.data = {"script_status": ScriptStatus.STARTED.codename}
        else:
            # check script progress or result
            cmd_result = subprocess.Popen(
                [cmd], shell=True, stdout=subprocess.PIPE
            ).communicate()[0]
            service_check_result.data = sort_data(cmd_result)

    except Exception as err:
        service_check_result.success = False
        service_check_result.message = repr(err)

    return service_check_result
