readonly CPU_LOAD_DATA="${BACKGROUND_DATA}/cpu_load_check.data";
readonly REASON_MYSQL="mysql";
readonly REASON_NONE="unknown";
readonly CPU_LOAD_CHECK_ERRORS="${BACKGROUND_DATA}/cpu_load_check.errors";
readonly CPU_LOAD_CASES=(check_if_mysql_is_top_process);

innodb_extra_ram(){
        local available_ram=$(free -b | awk '/Mem:/ {print $7}');
        echo $(( $available_ram - $available_ram / 5 ));
}

cpu_check_set_new_innodb_vars(){
        declare -A human_multipliers;
        human_multipliers=([K]=1000 [M]=1000000 [G]=1000000000);
        local innodb_vars="$@";
        local mysql_config_backup="${mysql_config}_$(date +%s)";
        local ram_total=$(free -b | awk '/Mem:/ {print $2}');
        rsync -q $mysql_config $mysql_config_backup;

        for i_var in $innodb_vars; do
                local var_name=$(cut -d= -f1 <<< $i_var);
                local multiplier=$(cut -d= -f2 <<< $i_var | grep -oP "[a-zA-Z]$" | tr [:lower:] [:upper:]);
                local i_value=$(cut -d= -f2 <<< $i_var | grep -oP "[0-9]+");
                local i_extra_ram;
                local i_value_new;

                if [[ $i_var =~ innodb_buffer_pool_size ]]; then
                        i_extra_ram=$(( $innodb_total_extra_ram - $innodb_total_extra_ram / 5 ));
                else
                        i_extra_ram=$(( $innodb_total_extra_ram / 5 ));
                fi;

                if [[ -n $i_value && $multiplier && ${human_multipliers[$multiplier]} ]]; then
                        i_value=$(( $i_value * ${human_multipliers[$multiplier]} ));
                fi;

                if [[ -z $i_value ]]; then
                        # For cases when innodb variable is not set
                        i_value_new=$i_extra_ram;
                        sed -i "s:\[mysqld\]:&\n$var_name=$i_value_new:" $mysql_config;
                elif [[ $var_name == "innodb_buffer_pool_size" && $i_value -ge $(( $ram_total - $ram_total / 4 )) ||
                                $var_name == "innodb_log_file_size" && $i_value -ge $(( $ram_total / 6 )) ]]; then
                        # An additional protection in case the current innodb variables are set too high
                        return 1;
                else
                        i_value_new=$(( $i_value + $i_extra_ram ));
                        sed -i "s/$i_var/$var_name=$i_value_new/" $mysql_config;
                fi;

                # Restore the mysql config backup if a value wasn't set in the mysql config file
                if [[ -z $(grep "^$var_name=$i_value_new$" $mysql_config) ]]; then
                        rsync -q $mysql_config_backup $mysql_config;
                        rm -f $mysql_config_backup;
                        return 1;
                fi;
        done;

        /scripts/restartsrv_mysql &>/dev/null && {
                rm -f $mysql_config_backup;
                return 0;
        } || {
                rsync -q $mysql_config_backup $mysql_config;
                rm -f $mysql_config_backup;
                return 1;
        }
}

mysql_top_process_complete_mgt(){
        local mysql_config="/etc/my.cnf";
        local innodb_buffer_pool_size=$(grep innodb_buffer_pool_size "$mysql_config" ||
                echo "innodb_buffer_pool_size=");
        local innodb_log_file_size=$(grep innodb_log_file_size "$mysql_config" || echo "innodb_log_file_size=");
        local innodb_total_extra_ram=$(innodb_extra_ram);
        local ram_usage_threshold=$(free -b | awk '/Mem:/ {printf "%d", ($2 * .2 )}') # 20 % of total ram

        # If available memory that can be used for innodb variables is less than 20% of total RAM, the optimization
        # can be skipped
        if [[ $innodb_total_extra_ram -gt $ram_usage_threshold ]]; then
                cpu_check_set_new_innodb_vars "$innodb_buffer_pool_size $innodb_log_file_size" && {
                        cpu_load_check_data[service_status]=$SERVICE_ACTIVE;
                        cpu_load_check_data[auto_fix_status]=$AUTO_FIX_SUCCESS;
                } || {
                        cpu_load_check_data[auto_fix_status]=$AUTO_FIX_FAIL;
                };
        else
                # innodb values cannot be optimized due to high memory usage
                cpu_load_check_data[recommendations]=$(printf "%s\n\n%s\n\n%s" \
                        "$(cpu_load_recommendation_mysql_optimization_unavailable)"\
                        "$(check_if_wp_is_installed_by_active_db)"\
                        "$(plan_upgrade_recommendation_if_optimization_fails)" | json_escape);
        fi;
}

ps_filter_cpu_usage() {
        readarray proc_arr <<< "$1";
        local minimal_threshold=9;

        for (( i = 0; $i < ${#proc_arr[@]}; ((++i)) )); do
                # Filter out the auto_report app process from the ps command results
                if [[ ${proc_arr[$i]} =~ $PROJECT_ROOT ]]; then
                        continue;
                fi;

                if [[ $(awk '{print $9}' <<< ${proc_arr[$i]} | cut -d. -f1) -gt $minimal_threshold ]]; then
                        echo "${proc_arr[$i]}" | sed -ze "s/\n$//";
                fi;
        done;
}

top_filter_cpu_usage() {
        readarray proc_arr <<< "$1";
        local minimal_threshold=1;

        for (( i = 0; $i < ${#proc_arr[@]}; ((++i)) )); do
                # Filter out the auto_report app process from the ps command results
                if [[ ${proc_arr[$i]} =~ $PROJECT_ROOT ]]; then
                        continue;
                fi;
                local proc_cpu_usage=$(awk '{print $9}' <<< ${proc_arr[$i]});
                if [[ $(awk '{if ($1 > $2) printf $1}' <<< "$proc_cpu_usage $minimal_threshold") ]]; then
                        echo "${proc_arr[$i]}" | sed -ze "s/\n$//";
                fi;
        done;
}

check_if_wp_is_installed_by_active_db(){
        local queries="$(for n in {0..30}; do
                mysql -e "show full processlist;" | grep -e Query -e Execute | grep -v -e NULL -e Progress; sleep 1;
        done)";
        local vhosts_root="$(cat /etc/userdatadomains | tr -s "=" | cut -d= -f5 | sed /^$/d | sort | uniq)";
        local db_names=$(tail -n +2 <<< "$queries" | tr -s '\t' ' ' | cut -d' ' -f4 | sort | uniq);
        local wp_websites="$(for vh_root in $vhosts_root; do
                for db in $db_names; do
                        grep -l "'$db'" ${vh_root}/wp-config.php 2> /dev/null 1>&2 && echo $vh_root && break;
                done;
        done)";

        if [[ -n $wp_websites ]]; then
                cpu_load_wordpress_mysql_recommendations;
        else
                cpu_load_generic_mysql_recommendation;
        fi;
}

mysql_top_process_basic_mgt(){
        local mtuner_rec="$(cpu_load_mysqltuner_recommendations)";
        local wp_rec="$(check_if_wp_is_installed_by_active_db)";
        local upgrade_rec="$(plan_upgrade_recommendation_if_optimization_fails)";

        if [[ -n $wp_rec ]]; then
                cpu_load_check_data[recommendations]=$(printf "%s\n\n%s\n\n%s\n" "$mtuner_rec" "$wp_rec" "$upgrade_rec" |
                        sed -z -e "s/\n/\\\n/g" -e "s/\t/\\\t/g");
        else
                cpu_load_check_data[recommendations]=$(printf "%s\n\n%s\n" "$mtuner_rec" "$upgrade_rec" |
                        sed -z -e "s/\n/\\\n/g" -e "s/\t/\\\t/g");
        fi;
}

check_if_mysql_is_top_process(){
        local top_proc=$(head -1 <<< "$processes_sorted");
        local mysql_match="mysql|maria";
        local mysql_cpu_usage=$(awk '{print $9}' <<< "$top_proc" | cut -d. -f1);
        local average_load=$(($VIRT_CORE_COUNT * 100 / 2));

        # The mysql process is the cause of the high CPU usage if it has over 50% cores used.
        if [[ $mysql_cpu_usage -ge $average_load ]]; then
                cpu_load_check_data[cpu_load_reason]="$REASON_MYSQL";
                cpu_load_check_data[ps_data]=$(printf "%s\n%s\n" "$(head -1 <<< "$processes")" "$top_proc" |
                json_escape);
                if [[ -n $(grep -iE "$mysql_match" <<< "$top_proc") && "$(management_type_check)" == $MANAGED ]]; then
                        mysql_top_process_basic_mgt;
                        return 0;
                elif [[ -n $(grep -iE "$mysql_match" <<< "$top_proc") && $(management_type_check) == $FULLY_MANAGED ]]; then
                        unset cpu_load_check_data[auto_fix_status];
                        mysql_top_process_complete_mgt;
                        return 0;
                else
                        return 1;
                fi;
        else
                # If the mysql process is using under %50 cores, then check for something else.
                return 1;
        fi;
}

cpu_threshold_check(){
        declare -A icinga_thresholds;
        # Icinga, per-core load thresholds
        #icinga_thresholds[1_min_warning]=1.1;
        #icinga_thresholds[5_min_warning]=0.9;
        #icinga_thresholds[15_min_warning]=0.7;
        icinga_thresholds[1_min_critical]=1.2;
        icinga_thresholds[5_min_critical]=1;
        icinga_thresholds[15_min_critical]=0.8;
        local virt_core_max_load=$(($(grep -ci processor /proc/cpuinfo) * 100));
        local load_average_1_min=$(awk '{printf "%d",($1 * 100)}' /proc/loadavg);
        local load_average_5_min=$(awk '{printf "%d",($2 * 100)}' /proc/loadavg);
        local load_average_15_min=$(awk '{printf "%d",($3 * 100)}' /proc/loadavg);

        # Converting the Icinga per-core thresholds into all-core load thresholds to match the uptime utility values
        #local la_1_min_warning_threshold=$(bc <<< "$virt_core_max_load * ${icinga_thresholds[1_min_warning]}" | cut -d. -f1);
        #local la_5_min_warning_threshold=$(bc <<< "$virt_core_max_load * ${icinga_thresholds[5_min_warning]}" | cut -d. -f1);
        #local la_15_min_warning_threshold=$(bc <<< "$virt_core_max_load * ${icinga_thresholds[15_min_warning]}" | cut -d. -f1);
        local la_1_min_critical_threshold=$(bc <<< "$virt_core_max_load * ${icinga_thresholds[1_min_critical]}" | cut -d. -f1);
        local la_5_min_critical_threshold=$(bc <<< "$virt_core_max_load * ${icinga_thresholds[5_min_critical]}" | cut -d. -f1);
        local la_15_min_critical_threshold=$(bc <<< "$virt_core_max_load * ${icinga_thresholds[15_min_critical]}" | cut -d. -f1);

        if [[ $load_average_1_min -ge $la_1_min_critical_threshold && $load_average_5_min -ge $la_5_min_critical_threshold
                && $load_average_15_min -ge $la_15_min_critical_threshold ]]; then
                        cpu_load_check_data[service_status]=$SERVICE_DOWN;
        #elif [[ $load_average_1_min -ge $la_1_min_warning_threshold && $load_average_5_min -ge $la_5_min_warning_threshold
        #        && $load_average_15_min -ge $la_15_min_warning_threshold ]]; then
        #                cpu_load_check_data[service_status]=$SERVICE_WARNING;
        else
                cpu_load_check_data[service_status]=$SERVICE_ACTIVE;
        fi;
}

cpu_load_check_main(){
        declare -A cpu_load_check_data;

        local top_data_full=$(top -bn1 -o %CPU);
        cpu_load_check_data[top_data]=$(echo "Total CPUs: $(grep -ci processor /proc/cpuinfo)" | json_escape;
                head -5 <<< "$top_data_full" | json_escape);
        local processes=$(tail -n +7 <<< "$top_data_full");
        local processes_sorted=$(top_filter_cpu_usage "$(tail -n +2 <<< "$processes")");

        cpu_threshold_check;
        if [[ ${cpu_load_check_data[service_status]} != $SERVICE_ACTIVE ]]; then
                for case in ${!CPU_LOAD_CASES[@]}; do
                        # Each cpu load check function should return 0 if the load reason is found and set the
                        # cpu_load_check_data[cpu_load_reason] value to describe a specific reason
                        if eval ${CPU_LOAD_CASES[$case]}; then
                                break;
                        fi;
                done;

                if [[ -z ${cpu_load_check_data[cpu_load_reason]} ]]; then
                        cpu_load_check_data[cpu_load_reason]="$REASON_NONE";
                        cpu_load_check_data[ps_data]=$(printf "%s\n%s\n" "$(head -1 <<< "$processes")" "$processes_sorted" |
                                json_escape);
                fi;
        fi;

        bash_arr_to_json cpu_load_check_data ${!cpu_load_check_data[@]} > "$CPU_LOAD_DATA"
}

cpu_load_check(){
        run_check_in_background "cpu_load_check_main" "$CPU_LOAD_DATA" "$CPU_LOAD_CHECK_ERRORS"\
                "${CHECK_LOCKS_DIR}/${FUNCNAME}.lock";
}